# Sample script template showing how to perform work during and after all rows have been seen

package MissingTracks 1.00;

use v5.16;
use strict;
use warnings;

our %callbacks = (
    perfile => \&do_perfile,
    post    => \&do_postprocessing,
);

my %tracks;

# Per-row callback function
#
# Do your per-file work here.
sub do_perfile {
    my ($cellval, $row) = @_;

    my $disc = $row->{'disc #'} || 0;
    $tracks{$cellval}{$disc}{$row->{'track #'}}++;

    return undef;
}

# Post-processing callback function
#
# This function will be called after all per-file processing has been completed.
# This is how you can do any post-processing work, if necessary.
sub do_postprocessing {

    my @out;
    for my $album (sort keys %tracks) {
	my $discstart = 1;
	for my $disc (sort {$a <=> $b} keys $tracks{$album}) {
	    #say "\tconsider disc: $disc";
	    if ($disc and $disc != $discstart) {
		my $lastd = $disc - 1;
		push @out, join '', "\tMISSING: disc ", $discstart == $lastd ? $lastd : "$discstart to $lastd";
		$discstart = $disc;
	    }
	    $discstart++;

	    my $trackstart = 1;
	    for my $track (sort {$a <=> $b} keys $tracks{$album}{$disc}) {
		#say "\t\tconsider track: $track";
		if ($track != $trackstart) {
		    my $last = $track - 1;
		    push @out, join '', "\tMISSING: disc $disc: track(s) ", $trackstart == $last ? $last : "$trackstart to $last";
		    $trackstart = $track;
		}
		$trackstart++;
	    }
	}
	if (@out) {
	    say "Album: $album";
	    say $_ for @out;
	    @out = ();
	}
    }

    return undef;
};

1;
